@@ -27,8 +27,8 @@ struct ParsedRegex
         Literal,
         AnyChar,
         AnyCharExceptNewLine,
-        Class,
-        CharacterType,
+        CharClass,
+        CharType,
         Sequence,
         Alternation,
         LineStart,
@@ -73,7 +73,7 @@ struct ParsedRegex
         };
     };
 
-    using NodeIndex = uint16_t;
+    using NodeIndex = int16_t;
     struct [[gnu::packed]] Node
     {
         Op op;
@@ -119,6 +119,7 @@ struct Children
         Index operator*() const { return m_pos; }
         bool operator!=(Sentinel) const { return m_pos != m_end; }
 
+    private:
         Index find_prev(Index parent, Index pos) const
         {
             Index child = parent+1;
@@ -179,11 +180,10 @@ private:
 
     NodeIndex disjunction(uint32_t capture = -1)
     {
-        NodeIndex index = new_node(ParsedRegex::Alternation);
-        get_node(index).value = capture;
+        NodeIndex index = add_node(ParsedRegex::Alternation, capture);
         while (true)
         {
-            alternative();
+            alternative(ParsedRegex::Sequence);
             if (at_end() or *m_pos != '|')
                 break;
             ++m_pos;
@@ -193,10 +193,10 @@ private:
         return index;
     }
 
-    NodeIndex alternative(ParsedRegex::Op op = ParsedRegex::Sequence)
+    NodeIndex alternative(ParsedRegex::Op op)
     {
-        NodeIndex index = new_node(op);
-        while (auto t = term())
+        NodeIndex index = add_node(op);
+        while (term())
         {}
         get_node(index).children_end = m_parsed_regex.nodes.size();
 
@@ -248,18 +248,18 @@ private:
 
         switch (*m_pos)
         {
-            case '^': ++m_pos; return new_node(ParsedRegex::LineStart);
-            case '$': ++m_pos; return new_node(ParsedRegex::LineEnd);
+            case '^': ++m_pos; return add_node(ParsedRegex::LineStart);
+            case '$': ++m_pos; return add_node(ParsedRegex::LineEnd);
             case '\\':
                 if (m_pos+1 == m_regex.end())
                     return {};
                 switch (*(m_pos+1))
                 {
-                    case 'b': m_pos += 2; return new_node(ParsedRegex::WordBoundary);
-                    case 'B': m_pos += 2; return new_node(ParsedRegex::NotWordBoundary);
-                    case 'A': m_pos += 2; return new_node(ParsedRegex::SubjectBegin);
-                    case 'z': m_pos += 2; return new_node(ParsedRegex::SubjectEnd);
-                    case 'K': m_pos += 2; return new_node(ParsedRegex::ResetStart);
+                    case 'b': m_pos += 2; return add_node(ParsedRegex::WordBoundary);
+                    case 'B': m_pos += 2; return add_node(ParsedRegex::NotWordBoundary);
+                    case 'A': m_pos += 2; return add_node(ParsedRegex::SubjectBegin);
+                    case 'z': m_pos += 2; return add_node(ParsedRegex::SubjectEnd);
+                    case 'K': m_pos += 2; return add_node(ParsedRegex::ResetStart);
                 }
                 break;
             case '(':
@@ -306,10 +306,7 @@ private:
         {
             case '.':
                 ++m_pos;
-                if (m_flags & Flags::DotMatchesNewLine)
-                    return new_node(ParsedRegex::AnyChar);
-                else
-                    return new_node(ParsedRegex::AnyCharExceptNewLine);
+                return add_node((m_flags & Flags::DotMatchesNewLine) ? ParsedRegex::AnyChar : ParsedRegex::AnyCharExceptNewLine);
             case '(':
             {
                 uint32_t capture_group = -1;
@@ -344,10 +341,10 @@ private:
             case '|': case ')':
                 return {};
             default:
-                if (contains("^$.*+?[]{}", cp) or (cp >= 0xF0000 and cp <= 0xFFFFF))
+                if (contains(StringView{"^$.*+?[]{}"}, cp) or (cp >= 0xF0000 and cp <= 0xFFFFF))
                     parse_error(format("unexpected '{}'", cp));
                 ++m_pos;
-                return new_node(ParsedRegex::Literal, cp);
+                return add_node(ParsedRegex::Literal, cp);
         }
     }
 
@@ -380,12 +377,12 @@ private:
 
         if (cp == 'Q')
         {
-            auto escaped_sequence = new_node(ParsedRegex::Sequence);
+            auto escaped_sequence = add_node(ParsedRegex::Sequence);
             constexpr StringView end_mark{"\\E"};
 
             auto quote_end = std::search(m_pos.base(), m_regex.end(), end_mark.begin(), end_mark.end());
             while (m_pos != quote_end)
-                new_node(ParsedRegex::Literal, *m_pos++);
+                add_node(ParsedRegex::Literal, *m_pos++);
             get_node(escaped_sequence).children_end = m_parsed_regex.nodes.size();
 
             if (quote_end != m_regex.end())
@@ -397,33 +394,33 @@ private:
         // CharacterClassEscape
         auto class_it = find_if(character_class_escapes, [cp](auto& c) { return c.cp == cp; });
         if (class_it != std::end(character_class_escapes))
-            return new_node(ParsedRegex::CharacterType, (Codepoint)class_it->ctype);
+            return add_node(ParsedRegex::CharType, (Codepoint)class_it->ctype);
 
         // CharacterEscape
         for (auto& control : control_escapes)
         {
             if (control.name == cp)
-                return new_node(ParsedRegex::Literal, control.value);
+                return add_node(ParsedRegex::Literal, control.value);
         }
 
         if (cp == '0')
-            return new_node(ParsedRegex::Literal, '\0');
+            return add_node(ParsedRegex::Literal, '\0');
         else if (cp == 'c')
         {
             if (at_end())
                 parse_error("unterminated control escape");
             Codepoint ctrl = *m_pos++;
             if (('a' <= ctrl and ctrl <= 'z') or ('A' <= ctrl and ctrl <= 'Z'))
-                return new_node(ParsedRegex::Literal, ctrl % 32);
+                return add_node(ParsedRegex::Literal, ctrl % 32);
             parse_error(format("Invalid control escape character '{}'", ctrl));
         }
         else if (cp == 'x')
-            return new_node(ParsedRegex::Literal, read_hex(2));
+            return add_node(ParsedRegex::Literal, read_hex(2));
         else if (cp == 'u')
-            return new_node(ParsedRegex::Literal, read_hex(6));
+            return add_node(ParsedRegex::Literal, read_hex(6));
 
         if (contains("^$\\.*+?()[]{}|", cp)) // SyntaxCharacter
-            return new_node(ParsedRegex::Literal, cp);
+            return add_node(ParsedRegex::Literal, cp);
         parse_error(format("unknown atom escape '{}'", cp));
     }
 
@@ -542,16 +539,18 @@ private:
         if (character_class.ctypes == CharacterType::None and not character_class.negative and
             character_class.ranges.size() == 1 and
             character_class.ranges.front().min == character_class.ranges.front().max)
-            return new_node(ParsedRegex::Literal, character_class.ranges.front().min);
+            return add_node(ParsedRegex::Literal, character_class.ranges.front().min);
 
         if (character_class.ctypes != CharacterType::None and not character_class.negative and
             character_class.ranges.empty())
-            return new_node(ParsedRegex::CharacterType, (Codepoint)character_class.ctypes);
+            return add_node(ParsedRegex::CharType, (Codepoint)character_class.ctypes);
 
-        auto class_id = m_parsed_regex.character_classes.size();
-        m_parsed_regex.character_classes.push_back(std::move(character_class));
+        auto it = std::find(m_parsed_regex.character_classes.begin(), m_parsed_regex.character_classes.end(), character_class);
+        auto class_id = it - m_parsed_regex.character_classes.begin();
+        if (it == m_parsed_regex.character_classes.end())
+            m_parsed_regex.character_classes.push_back(std::move(character_class));
 
-        return new_node(ParsedRegex::Class, class_id);
+        return add_node(ParsedRegex::CharClass, class_id);
     }
 
     ParsedRegex::Quantifier quantifier()
@@ -604,8 +603,7 @@ private:
         }
     }
 
-    NodeIndex new_node(ParsedRegex::Op op, Codepoint value = -1,
-                          ParsedRegex::Quantifier quantifier = {ParsedRegex::Quantifier::One})
+    NodeIndex add_node(ParsedRegex::Op op, Codepoint value = -1, ParsedRegex::Quantifier quantifier = {ParsedRegex::Quantifier::One})
     {
         constexpr auto max_nodes = std::numeric_limits<int16_t>::max();
         const NodeIndex res = m_parsed_regex.nodes.size();
@@ -616,13 +614,12 @@ private:
         return res;
     }
 
-    bool at_end() const { return m_pos == m_regex.end(); }
-
     ParsedRegex::Node& get_node(NodeIndex index)
     {
         return m_parsed_regex.nodes[index];
     }
 
+    bool at_end() const { return m_pos == m_regex.end(); }
 
     [[gnu::noreturn]]
     void parse_error(StringView error) const
@@ -638,8 +635,8 @@ private:
         for (auto child_index : Children<>{m_parsed_regex, index})
         {
             auto& child = get_node(child_index);
-            if (child.op != ParsedRegex::Literal and child.op != ParsedRegex::Class and
-                child.op != ParsedRegex::CharacterType and child.op != ParsedRegex::AnyChar and
+            if (child.op != ParsedRegex::Literal and child.op != ParsedRegex::CharClass and
+                child.op != ParsedRegex::CharType and child.op != ParsedRegex::AnyChar and
                 child.op != ParsedRegex::AnyCharExceptNewLine)
                 parse_error("Lookaround can only contain literals, any chars or character classes");
             if (child.op == ParsedRegex::Literal and
@@ -684,6 +681,8 @@ constexpr RegexParser::ControlEscape RegexParser::control_escapes[];
 
 struct RegexCompiler
 {
+    using OpIndex = int16_t;
+
     RegexCompiler(ParsedRegex&& parsed_regex, RegexCompileFlags flags)
         : m_flags(flags), m_parsed_regex{parsed_regex}
     {
@@ -722,7 +721,7 @@ struct RegexCompiler
 private:
 
     template<RegexMode direction>
-    uint32_t compile_node_inner(ParsedRegex::NodeIndex index)
+    OpIndex compile_node_inner(ParsedRegex::NodeIndex index)
     {
         auto& node = get_node(index);
 
@@ -733,16 +732,13 @@ private:
                           (node.value == 0 or (node.value != -1 and not (m_flags & RegexCompileFlags::NoSubs)));
         constexpr bool forward = direction == RegexMode::Forward;
         if (save)
-            push_inst(CompiledRegex::Save, node.value * 2 + (forward ? 0 : 1));
+            push_inst(CompiledRegex::Save, {.save_index = int16_t(node.value * 2 + (forward ? 0 : 1))});
 
         Vector<uint32_t> goto_inner_end_offsets;
         switch (node.op)
         {
             case ParsedRegex::Literal:
-                if (ignore_case)
-                    push_inst(CompiledRegex::Literal_IgnoreCase, to_lower(node.value));
-                else
-                    push_inst(CompiledRegex::Literal, node.value);
+                push_inst(CompiledRegex::Literal, {.literal={.codepoint=ignore_case ? to_lower(node.value) : node.value, .ignore_case=ignore_case}});
                 break;
             case ParsedRegex::AnyChar:
                 push_inst(CompiledRegex::AnyChar);
@@ -750,11 +746,11 @@ private:
             case ParsedRegex::AnyCharExceptNewLine:
                 push_inst(CompiledRegex::AnyCharExceptNewLine);
                 break;
-            case ParsedRegex::Class:
-                push_inst(CompiledRegex::Class, node.value);
+            case ParsedRegex::CharClass:
+                push_inst(CompiledRegex::CharClass, {.character_class_index=int16_t(node.value)});
                 break;
-            case ParsedRegex::CharacterType:
-                push_inst(CompiledRegex::CharacterType, node.value);
+            case ParsedRegex::CharType:
+                push_inst(CompiledRegex::CharType, {.character_type=CharacterType{(unsigned char)node.value}});
                 break;
             case ParsedRegex::Sequence:
             {
@@ -764,19 +760,19 @@ private:
             }
             case ParsedRegex::Alternation:
             {
-                auto split_pos = m_program.instructions.size();
                 for (auto child : Children<>{m_parsed_regex, index})
                 {
                     if (child != index+1)
-                        push_inst(CompiledRegex::Split_PrioritizeParent);
+                        push_inst(CompiledRegex::Split);
                 }
+                auto split_pos = m_program.instructions.size();
 
                 const auto end = node.children_end;
                 for (auto child : Children<>{m_parsed_regex, index})
                 {
                     auto node = compile_node<direction>(child);
                     if (child != index+1)
-                        m_program.instructions[split_pos++].param = node;
+                        m_program.instructions[--split_pos].param.split = CompiledRegex::Param::Split{.target = node, .prioritize_parent = true};
                     if (get_node(child).children_end != end)
                     {
                         auto jump = push_inst(CompiledRegex::Jump);
@@ -786,71 +782,66 @@ private:
                 break;
             }
             case ParsedRegex::LookAhead:
-                push_inst(ignore_case ? CompiledRegex::LookAhead_IgnoreCase
-                                      : CompiledRegex::LookAhead,
-                          push_lookaround<RegexMode::Forward>(index, ignore_case));
-                break;
             case ParsedRegex::NegativeLookAhead:
-                push_inst(ignore_case ? CompiledRegex::NegativeLookAhead_IgnoreCase
-                                      : CompiledRegex::NegativeLookAhead,
-                          push_lookaround<RegexMode::Forward>(index, ignore_case));
+                push_inst(CompiledRegex::LookAround, {.lookaround={
+                    .index=push_lookaround<RegexMode::Forward>(index, ignore_case),
+                    .ahead=true,
+                    .positive=node.op == ParsedRegex::LookAhead,
+                    .ignore_case=ignore_case}});
                 break;
             case ParsedRegex::LookBehind:
-                push_inst(ignore_case ? CompiledRegex::LookBehind_IgnoreCase
-                                      : CompiledRegex::LookBehind,
-                          push_lookaround<RegexMode::Backward>(index, ignore_case));
-                break;
             case ParsedRegex::NegativeLookBehind:
-                push_inst(ignore_case ? CompiledRegex::NegativeLookBehind_IgnoreCase
-                                      : CompiledRegex::NegativeLookBehind,
-                          push_lookaround<RegexMode::Backward>(index, ignore_case));
+                push_inst(CompiledRegex::LookAround, {.lookaround={
+                    .index=push_lookaround<RegexMode::Backward>(index, ignore_case),
+                    .ahead=false,
+                    .positive=node.op == ParsedRegex::LookBehind,
+                    .ignore_case=ignore_case}});
                 break;
             case ParsedRegex::LineStart:
-                push_inst(CompiledRegex::LineStart);
+                push_inst(CompiledRegex::LineAssertion, {.line_start=true});
                 break;
             case ParsedRegex::LineEnd:
-                push_inst(CompiledRegex::LineEnd);
+                push_inst(CompiledRegex::LineAssertion, {.line_start=false});
                 break;
             case ParsedRegex::WordBoundary:
-                push_inst(CompiledRegex::WordBoundary);
+                push_inst(CompiledRegex::WordBoundary, {.word_boundary_positive=true});
                 break;
             case ParsedRegex::NotWordBoundary:
-                push_inst(CompiledRegex::NotWordBoundary);
+                push_inst(CompiledRegex::WordBoundary, {.word_boundary_positive=false});
                 break;
             case ParsedRegex::SubjectBegin:
-                push_inst(CompiledRegex::SubjectBegin);
+                push_inst(CompiledRegex::SubjectAssertion, {.subject_begin=true});
                 break;
             case ParsedRegex::SubjectEnd:
-                push_inst(CompiledRegex::SubjectEnd);
+                push_inst(CompiledRegex::SubjectAssertion, {.subject_begin=false});
                 break;
             case ParsedRegex::ResetStart:
-                push_inst(CompiledRegex::Save, 0);
+                push_inst(CompiledRegex::Save, {.save_index=0});
                 break;
         }
 
         for (auto& offset : goto_inner_end_offsets)
-            m_program.instructions[offset].param = m_program.instructions.size();
+            m_program.instructions[offset].param.jump_target = m_program.instructions.size();
 
         if (save)
-            push_inst(CompiledRegex::Save, node.value * 2 + (forward ? 1 : 0));
+            push_inst(CompiledRegex::Save, {.save_index=int16_t(node.value * 2 + (forward ? 1 : 0))});
 
         return start_pos;
     }
 
     template<RegexMode direction>
-    uint32_t compile_node(ParsedRegex::NodeIndex index)
+    OpIndex compile_node(ParsedRegex::NodeIndex index)
     {
         auto& node = get_node(index);
 
-        const uint32_t start_pos = (uint32_t)m_program.instructions.size();
-        Vector<uint32_t> goto_ends;
+        const OpIndex start_pos = (OpIndex)m_program.instructions.size();
+        Vector<OpIndex> goto_ends;
 
         auto& quantifier = node.quantifier;
 
         if (quantifier.allows_none())
         {
-            auto split_pos = push_inst(quantifier.greedy ? CompiledRegex::Split_PrioritizeParent
-                                                         : CompiledRegex::Split_PrioritizeChild);
+            auto split_pos = push_inst(CompiledRegex::Split, {.split={.target=0, .prioritize_parent=quantifier.greedy}});
             goto_ends.push_back(split_pos);
         }
 
@@ -860,41 +851,38 @@ private:
             inner_pos = compile_node_inner<direction>(index);
 
         if (quantifier.allows_infinite_repeat())
-            push_inst(quantifier.greedy ? CompiledRegex::Split_PrioritizeChild
-                                        : CompiledRegex::Split_PrioritizeParent,
-                      inner_pos);
+            push_inst(CompiledRegex::Split, {.split = {.target=inner_pos, .prioritize_parent=not quantifier.greedy}});
         // Write the node as an optional match for the min -> max counts
         else for (int i = std::max((int16_t)1, quantifier.min); // STILL UGLY !
                   i < quantifier.max; ++i)
         {
-            auto split_pos = push_inst(quantifier.greedy ? CompiledRegex::Split_PrioritizeParent
-                                                         : CompiledRegex::Split_PrioritizeChild);
+            auto split_pos = push_inst(CompiledRegex::Split, {.split={.target=0, .prioritize_parent=quantifier.greedy}});
             goto_ends.push_back(split_pos);
             compile_node_inner<direction>(index);
         }
 
         for (auto offset : goto_ends)
-            m_program.instructions[offset].param = m_program.instructions.size();
+            m_program.instructions[offset].param.split.target = m_program.instructions.size();
 
         return start_pos;
     }
 
-    uint32_t push_inst(CompiledRegex::Op op, uint32_t param = 0)
+    OpIndex push_inst(CompiledRegex::Op op, CompiledRegex::Param param = {})
     {
-        constexpr auto max_instructions = std::numeric_limits<int16_t>::max();
-        const uint32_t res = m_program.instructions.size();
-        if (res > max_instructions)
+        constexpr auto max_instructions = std::numeric_limits<OpIndex>::max();
+        const auto res = m_program.instructions.size();
+        if (res >= max_instructions)
             throw regex_error(format("regex compiled to more than {} instructions", max_instructions));
         m_program.instructions.push_back({ op, false, 0, param });
-        return res;
+        return OpIndex(res);
     }
 
     template<RegexMode direction>
-    uint32_t push_lookaround(ParsedRegex::NodeIndex index, bool ignore_case)
+    int16_t push_lookaround(ParsedRegex::NodeIndex index, bool ignore_case)
     {
         using Lookaround = CompiledRegex::Lookaround;
 
-        const uint32_t res = m_program.lookarounds.size();
+        const int16_t res = m_program.lookarounds.size();
         for (auto child : Children<direction>{m_parsed_regex, index})
         {
             auto& character = get_node(child);
@@ -905,9 +893,9 @@ private:
                 m_program.lookarounds.push_back(Lookaround::AnyChar);
             else if (character.op == ParsedRegex::AnyCharExceptNewLine)
                 m_program.lookarounds.push_back(Lookaround::AnyCharExceptNewLine);
-            else if (character.op == ParsedRegex::Class)
+            else if (character.op == ParsedRegex::CharClass)
                 m_program.lookarounds.push_back(static_cast<Lookaround>(to_underlying(Lookaround::CharacterClass) + character.value));
-            else if (character.op == ParsedRegex::CharacterType)
+            else if (character.op == ParsedRegex::CharType)
                 m_program.lookarounds.push_back(static_cast<Lookaround>(to_underlying(Lookaround::CharacterType) | character.value));
             else
                 kak_assert(false);
@@ -951,7 +939,7 @@ private:
                         start_desc.map[cp] = true;
                 }
                return node.quantifier.allows_none();
-            case ParsedRegex::Class:
+            case ParsedRegex::CharClass:
             {
                 auto& character_class = m_parsed_regex.character_classes[node.value];
                 if (character_class.ctypes == CharacterType::None and
@@ -978,7 +966,7 @@ private:
                 start_desc.map[CompiledRegex::StartDesc::other] = true;
                 return node.quantifier.allows_none();
             }
-            case ParsedRegex::CharacterType:
+            case ParsedRegex::CharType:
             {
                 const CharacterType ctype = (CharacterType)node.value;
                 for (Codepoint cp = 0; cp < CompiledRegex::StartDesc::count; ++cp)
@@ -1041,12 +1029,12 @@ private:
         if (not (m_flags & RegexCompileFlags::Optimize))
             return;
 
-        auto is_jump = [](CompiledRegex::Op op) { return op >= CompiledRegex::Op::Jump and op <= CompiledRegex::Op::Split_PrioritizeChild; };
+        auto is_jump = [](CompiledRegex::Op op) { return op >= CompiledRegex::Op::Jump and op <= CompiledRegex::Op::Split; };
         for (auto i = begin; i < end; ++i)
         {
             auto& inst = m_program.instructions[i];
             if (is_jump(inst.op))
-                m_program.instructions[inst.param].last_step = 0xffff; // tag as jump target
+                m_program.instructions[inst.param.jump_target].last_step = 0xffff; // tag as jump target
         }
 
         for (auto block_begin = begin; block_begin < end; )
@@ -1064,7 +1052,7 @@ private:
     void peephole_optimize(size_t begin, size_t end)
     {
         // Move saves after all assertions on the same character
-        auto is_assertion = [](CompiledRegex::Op op) { return op >= CompiledRegex::LineStart; };
+        auto is_assertion = [](CompiledRegex::Op op) { return op >= CompiledRegex::LineAssertion; };
         for (auto i = begin, j = begin + 1; j < end; ++i, ++j)
         {
             if (m_program.instructions[i].op == CompiledRegex::Save and
@@ -1095,10 +1083,7 @@ String dump_regex(const CompiledRegex& program)
         switch (inst.op)
         {
             case CompiledRegex::Literal:
-                res += format("literal {}\n", inst.param);
-                break;
-            case CompiledRegex::Literal_IgnoreCase:
-                res += format("literal (ignore case) {}\n", inst.param);
+                res += format("literal {}{}\n", inst.param.literal.ignore_case ? "(ignore case) " : "", inst.param.literal.codepoint);
                 break;
             case CompiledRegex::AnyChar:
                 res += "any char\n";
@@ -1107,73 +1092,44 @@ String dump_regex(const CompiledRegex& program)
                 res += "anything but newline\n";
                 break;
             case CompiledRegex::Jump:
-                res += format("jump {}\n", inst.param);
+                res += format("jump {}\n", inst.param.jump_target);
                 break;
-            case CompiledRegex::Split_PrioritizeParent:
-            case CompiledRegex::Split_PrioritizeChild:
+            case CompiledRegex::Split:
             {
                 res += format("split (prioritize {}) {}\n",
-                              inst.op == CompiledRegex::Split_PrioritizeParent ? "parent" : "child",
-                              inst.param);
+                              (inst.param.split.prioritize_parent) ? "parent" : "child",
+                              inst.param.split.target);
                 break;
             }
             case CompiledRegex::Save:
-                res += format("save {}\n", inst.param);
+                res += format("save {}\n", inst.param.save_index);
                 break;
-            case CompiledRegex::Class:
-                res += format("class {}\n", inst.param);
+            case CompiledRegex::CharClass:
+                res += format("character class {}\n", inst.param.character_class_index);
                 break;
-            case CompiledRegex::CharacterType:
-                res += format("character type {}\n", inst.param);
+            case CompiledRegex::CharType:
+                res += format("character type {}\n", to_underlying(inst.param.character_type));
                 break;
-            case CompiledRegex::LineStart:
-                res += "line start\n";
+            case CompiledRegex::LineAssertion:
+                res += format("line {}\n", inst.param.line_start ? "start" : "end");;
                 break;
-            case CompiledRegex::LineEnd:
-                res += "line end\n";
+            case CompiledRegex::SubjectAssertion:
+                res += format("subject {}\n", inst.param.subject_begin ? "begin" : "end");
                 break;
             case CompiledRegex::WordBoundary:
-                res += "word boundary\n";
+                res += format("{}word boundary\n", inst.param.word_boundary_positive ? "" : "not ");
                 break;
-            case CompiledRegex::NotWordBoundary:
-                res += "not word boundary\n";
-                break;
-            case CompiledRegex::SubjectBegin:
-                res += "subject begin\n";
-                break;
-            case CompiledRegex::SubjectEnd:
-                res += "subject end\n";
-                break;
-            case CompiledRegex::LookAhead:
-            case CompiledRegex::NegativeLookAhead:
-            case CompiledRegex::LookBehind:
-            case CompiledRegex::NegativeLookBehind:
-            case CompiledRegex::LookAhead_IgnoreCase:
-            case CompiledRegex::NegativeLookAhead_IgnoreCase:
-            case CompiledRegex::LookBehind_IgnoreCase:
-            case CompiledRegex::NegativeLookBehind_IgnoreCase:
+            case CompiledRegex::LookAround:
             {
-                const char* name = nullptr;
-                if (inst.op == CompiledRegex::LookAhead)
-                    name = "look ahead";
-                if (inst.op == CompiledRegex::NegativeLookAhead)
-                    name = "negative look ahead";
-                if (inst.op == CompiledRegex::LookBehind)
-                    name = "look behind";
-                if (inst.op == CompiledRegex::NegativeLookBehind)
-                    name = "negative look behind";
-
-                if (inst.op == CompiledRegex::LookAhead_IgnoreCase)
-                    name = "look ahead (ignore case)";
-                if (inst.op == CompiledRegex::NegativeLookAhead_IgnoreCase)
-                    name = "negative look ahead (ignore case)";
-                if (inst.op == CompiledRegex::LookBehind_IgnoreCase)
-                    name = "look behind (ignore case)";
-                if (inst.op == CompiledRegex::NegativeLookBehind_IgnoreCase)
-                    name = "negative look behind (ignore case)";
+                String name;
+                name += inst.param.lookaround.positive ? "" : "negative ";
+                name += "look ";
+                name += inst.param.lookaround.ahead ? "ahead " : "behind ";
+                if (inst.param.lookaround.ignore_case)
+                    name += " (ignore case)";
 
                 String str;
-                for (auto it = program.lookarounds.begin() + inst.param;
+                for (auto it = program.lookarounds.begin() + inst.param.lookaround.index;
                      *it != CompiledRegex::Lookaround::EndOfLookaround; ++it)
                     utf8::dump(std::back_inserter(str), to_underlying(*it));
                 res += format("{} ({})\n", name, str);
@@ -1214,27 +1170,24 @@ bool is_character_class(const CharacterClass& character_class, Codepoint cp)
     if (character_class.ignore_case)
         cp = to_lower(cp);
 
-    auto it = std::lower_bound(character_class.ranges.begin(),
-                               character_class.ranges.end(), cp,
-                               [](auto& range, Codepoint cp)
-                               { return range.max < cp; });
-
-    auto found = (it != character_class.ranges.end() and it->min <= cp) or
-                 is_ctype(character_class.ctypes, cp);
+    auto it = std::find_if(character_class.ranges.begin(),
+                           character_class.ranges.end(),
+                           [cp](auto& range) { return range.min <= cp and cp <= range.max; });
 
+    bool found = it != character_class.ranges.end() or (character_class.ctypes != CharacterType::None and
+                                                        is_ctype(character_class.ctypes, cp));
     return found != character_class.negative;
 }
 
 bool is_ctype(CharacterType ctype, Codepoint cp)
 {
-    return ((ctype & CharacterType::Whitespace)              and     is_blank(cp))            or
-           ((ctype & CharacterType::HorizontalWhitespace)    and     is_horizontal_blank(cp)) or
-           ((ctype & CharacterType::Digit)                   and     iswdigit(cp))            or
-           ((ctype & CharacterType::Word)                    and     is_word(cp))             or
-           ((ctype & CharacterType::NotWhitespace)           and not is_blank(cp))            or
-           ((ctype & CharacterType::NotHorizontalWhitespace) and not is_horizontal_blank(cp)) or
-           ((ctype & CharacterType::NotDigit)                and not iswdigit(cp))            or
-           ((ctype & CharacterType::NotWord)                 and not is_word(cp));
+    auto check = [&](CharacterType bit, CharacterType not_bit, auto&& func) {
+        return (ctype & (bit | not_bit)) and func(cp) == (bool)(ctype & bit);
+    };
+    return check(CharacterType::Word, CharacterType::NotWord, [](Codepoint cp) { return is_word(cp); }) or
+           check(CharacterType::Whitespace, CharacterType::NotWhitespace, is_blank) or
+           check(CharacterType::HorizontalWhitespace, CharacterType::NotHorizontalWhitespace, is_horizontal_blank) or
+           check(CharacterType::Digit, CharacterType::NotDigit, iswdigit);
 }
 
 namespace
@@ -1400,7 +1353,11 @@ auto test_regex = UnitTest{[]{
         kak_assert(StringView{vm.captures()[0], vm.captures()[1]} == "bar");
         kak_assert(not vm.exec("bar"));
     }
-
+    {
+        TestVM<RegexMode::Forward | RegexMode::Search> vm{R"(foobaz|foo|foobar)"};
+        kak_assert(vm.exec("foobar"));
+        kak_assert(StringView{vm.captures()[0], vm.captures()[1]} == "foo");
+    }
     {
         TestVM<RegexMode::Forward> vm{R"((fo+?).*)"};
         kak_assert(vm.exec("foooo"));
@@ -1582,6 +1539,12 @@ auto test_regex = UnitTest{[]{
         kak_assert(vm.exec("\t\n\v\f\r"));
     }
 
+    {
+        TestVM<> vm{R"([\t-\r]\h+[\t-\r])"};
+        kak_assert(vm.character_classes.size() == 1);
+        kak_assert(vm.exec("\n  \f"));
+    }
+
     {
         TestVM<> vm{R"([^\x00-\x7F]+)"};
         kak_assert(not vm.exec("ascii"));
